<?php

namespace Drp\AdfsAuthentication;

use Carbon\Carbon;
use Drp\AdfsAuthentication\Exceptions\AdfsServerException;
use GuzzleHttp\Client;
use GuzzleHttp\Exception\ClientException;
use GuzzleHttp\Exception\ServerException;
use Lcobucci\JWT\Parser;
use Lcobucci\JWT\Token;
use League\OAuth2\Server\Exception\OAuthServerException;
use Mockery\Exception;
use stdClass;

class AdfsHandler
{
    /**
     * @var Client
     */
    protected $client;

    /**
     * ADFSHandler constructor.
     * @param Client $client
     */
    public function __construct(Client $client)
    {
        $this->client = $client;
    }

    /**
     * Send request to ADFS Authentication Server
     * and handle responses.
     *
     * @param $username
     * @param $password
     * @return mixed
     * @throws AdfsServerException
     * @throws OAuthServerException
     */
    public function authenticate($username, $password)
    {
        $response = $this->client->request(
            'POST',
            config('adfs.adfs_url') . '/oauth2/token',
            ['form_params' => [
                'client_id' => config('adfs.adfs_client_id'),
                'client_secret' => config('adfs.adfs_client_secret'),
                'grant_type' => 'password',
                'username' => $username,
                'password' => $password,
                'scope' => 'openid'
            ]]
        );

        $contents = json_decode($response->getBody()->getContents());

        if (!json_last_error()) {
            return $contents;
        }

        throw AdfsServerException::invalidResponse();
    }

    /**
     * Store oAuth Tokens in the database.
     *
     * @param stdClass $response
     * @param integer $userId
     * @return void
     */
    public function storeTokens(stdClass $response, $userId)
    {
        AdfsOAuthToken::updateOrCreate(
            [
                'user_id' => $userId
            ],
            [
                'access_token' => $response->access_token,
                'access_token_expiry' => $this->calculateExpiry($response->expires_in),
                'refresh_token' => $response->refresh_token,
                'refresh_token_expiry' => $this->calculateExpiry($response->refresh_token_expires_in),
                'id_token' => $response->id_token,
                'user_id' => $userId
            ]
        );
    }

    /**
     * Refresh Token Command Endpoint
     *
     * @return void
     */
    public function refreshAccessTokens()
    {
        $expiringSoonTokens = AdfsOAuthToken::expiringSoon()->get();

        $expiringSoonTokens->each(function ($item) {
            try {
                $response = $this->refreshToken($item->refresh_token);
                $item->updateAccessToken($response->access_token, $this->calculateExpiry($response->expires_in));
            } catch (\Exception $e) {
                return;
            }
        });
    }

    /**
     * Refresh the tokens requested.
     *
     * @param $token
     * @return mixed
     * @throws AdfsServerException
     * @throws OAuthServerException
     */
    public function refreshToken($token)
    {
        try {
            $response = $this->client->post(
                config('adfs.adfs_url') . '/oauth2/token',
                ['form_params' => [
                    'client_id' => config('adfs.adfs_client_id'),
                    'client_secret' => config('adfs.adfs_client_secret'),
                    'grant_type' => 'refresh_token',
                    'refresh_token' => $token,
                    'scope' => 'openid'
                ]]
            );

            $contents = json_decode($response->getBody()->getContents());

            if (!json_last_error()) {
                return $contents;
            }

            throw AdfsServerException::invalidResponse();
        } catch (ClientException $e) {
            throw OAuthServerException::invalidRefreshToken();
        } catch (ServerException $e) {
            throw AdfsServerException::serverError();
        } catch (Exception $e) {
            throw AdfsServerException::unknownError();
        }
    }

    /**
     * Calculate the token expiry date based on the current
     * date/time and the expiry_in provided.
     *
     * @param integer $expiresIn
     * @return string
     */
    public function calculateExpiry($expiresIn)
    {
        return Carbon::now()->addSeconds($expiresIn)->toDateTimeString();
    }

    /**
     * Decode the id token received back
     * from the ADFS server.
     *
     * @param string $idToken
     * @return Token
     */
    public function decodeIdToken($idToken): Token
    {
        $idToken = (new Parser())->parse((string)$idToken);

        return $idToken;
    }
}
