<?php

namespace Drp\AdfsAuthentication;

use Drp\AdfsAuthentication\Exceptions\AdfsServerException;
use Drp\AdfsAuthentication\Exceptions\InvalidIdTokenException;
use Illuminate\Auth\Authenticatable;
use Illuminate\Auth\EloquentUserProvider as UserProvider;
use Illuminate\Contracts\Auth\Authenticatable as UserContract;
use Illuminate\Contracts\Hashing\Hasher as HasherContract;

/**
 * Class AdfsUserProvider
 * @package App\Providers
 */
class AdfsUserProvider extends UserProvider
{
    /**
     * @var adfsHandler
     */
    protected $handler;

    /**
     * AdfsUserProvider constructor.
     * @param HasherContract $hasher
     * @param string $model
     * @param adfsHandler $handler
     */
    public function __construct(HasherContract $hasher, $model, adfsHandler $handler)
    {
        parent::__construct($hasher, $model);
        $this->handler = $handler;
    }

    /**
     * @param UserContract $user
     * @param array $credentials
     * @return bool
     * @SuppressWarnings(PHPMD)
     */
    public function validateCredentials(UserContract $user, array $credentials)
    {
        return true;
    }

    /**
     * Checks Credentials are correct by sending a request to ADFS
     * Create / Retrieves the User
     * Stores the Authentication Tokens
     *
     * @param array $credentials
     * @throws InvalidIdTokenException
     * @throws AdfsServerException
     * @return Authenticatable|null
     */
    public function retrieveByCredentials(array $credentials)
    {
        $response = $this->handler->authenticate($credentials['username'], $credentials['password']);
        $user = $this->createNewUser($response->id_token);
        $this->handler->storeTokens($response, $user->id);
        return $user;
    }

    /**
     * Create / Retrieves the User based on the ID Token
     * received from the ADFS Server.
     *
     * @param string $idToken
     * @throws InvalidIdTokenException Invalid Id Token Received.
     * @return Authenticatable|boolean
     */
    public function createNewUser($idToken)
    {
        try {
            $userInformation = $this->handler->decodeIdToken($idToken);
            $emailAddress = $userInformation->getClaim('upn');
            $name = $this->getNameFromEmailAddress($emailAddress);
        } catch (\RuntimeException $e) {
            throw InvalidIdTokenException::invalidToken();
        }

        return \App\User::firstOrCreate(['email' => $emailAddress], ['name' => $name]);
    }

    /**
     * @param $emailAddress
     * @return string
     */
    public function getNameFromEmailAddress($emailAddress)
    {
        return ucwords(str_replace('.', ' ', explode('@', $emailAddress)[0]));
    }
}
